@@ -1,7 +1,7 @@
 import re, weakref, logging
 from copy import copy
 from warnings import warn
-from itertools import ifilter, count, chain, izip, islice
+from itertools import ifilter, count, chain, izip, islice, ifilterfalse
 from inspect import isclass
 
 import tw
@@ -215,10 +215,10 @@ class Widget(view.Renderable):
 
 
     def path(self):
-        yield self
-        if self.parent:
-            for i in self.parent.path:
-                yield i
+        item = self
+        while item:
+            yield item
+            item = item.parent
     path = property(path, doc="""\
         Iterates a walk from this widget to the root of the tree
         """)
@@ -323,7 +323,7 @@ class Widget(view.Renderable):
             ('foo', 3)
 
         """
-        log.debug("Cloning %r", self)
+        #log.debug("Cloning %r", self)
         return Child(
             self.__class__, self._id, children=self.children, **self.orig_kw
             )(*args, **kw)
@@ -334,7 +334,7 @@ class Widget(view.Renderable):
         cls = self.__class__
         new_name = 'Repeated'+cls.__name__
         new_class = type(new_name, (RepeatedWidget, cls), {})
-        log.debug("Generating %r for repeating %r", new_class, self)
+        #log.debug("Generating %r for repeating %r", new_class, self)
         return Child(
             new_class, self._id, children=self.children, **self.orig_kw
             )(*args, **kw)
@@ -383,16 +383,16 @@ class Widget(view.Renderable):
         perform final setup after the widget is initialized but before it is
         locked.
         """
-        self._collect_resources() 
         if len(self._js_calls) > 0:
             from tw.core.resources import JSFunctionCalls
-            log.debug("Creating JSFunctionCalls for %r. Functions: %s",
-                self, self._js_calls,
-                )
-            self._resources.add(
+            #log.debug("Creating JSFunctionCalls for %r. Functions: %s",
+                #self, self._js_calls,
+                #)
+            self.javascript.append(
                 JSFunctionCalls(function_calls=self._js_calls)
                 )
-        log.debug("Finished initializing %r", self)
+        self._collect_resources() 
+        #log.debug("Finished initializing %r", self)
         assert not self._is_initialized
         self._is_initialized = True
         assert not self._is_locked
@@ -439,15 +439,12 @@ class Widget(view.Renderable):
         widget is rendered.
         """
         if self._is_initialized:
-            log.debug("Adding call <%s> for %r dynamically.", call, self)
-            # Inject the widget that will render the calls
+            #log.debug("Adding call <%s> for %r dynamically.", call, self)
             from tw.core.resources import dynamic_js_calls
             dynamic_js_calls.inject_call(call, location)
-        elif not self._is_initialized:
-            log.debug("Adding call <%s> for %r statically.", call, self)
-            self._js_calls.append(call)
         else:
-            raise TypeError("%r does not allow dynamic_calls." % self)
+            #log.debug("Adding call <%s> for %r statically.", call, self)
+            self._js_calls.append(str(call))
             
 
 
@@ -493,20 +490,29 @@ class Widget(view.Renderable):
                 else:
                     value[k] = v
         # If we have children, create a dict from the attributes of value
-        elif len(self.children) > 0 and not isinstance(value, (dict,list)):
+        elif len(self.children_deep) > 0 and not isinstance(value, (dict,list)):
             value = dict([
                 (w._id, getattr(value, w._id)) 
-                    for w in self.children if hasattr(value, w._id)
+                    for w in self.children_deep if hasattr(value, w._id)
                 ])
         return value
 
+    def register_resources(self):
+        """
+        Register the resources required by this Widget with
+        :attr:`tw.framework` for inclusion in the page.
+
+        This method is called whenever a :class:`Widget` is rendered
+        """
+        if self.is_root:
+            tw.framework.register_resources(self.retrieve_resources())
+
     def render(self, value=None, **kw):
         """
         Renders a widget as an unicode string.
         """
         kw = self.prepare_dict(value, kw)
-        if self.is_root:
-            tw.framework.register_resources(self.retrieve_resources())
+        self.register_resources()
         return super(Widget, self).render(**kw)
 
     def display(self, value=None, **kw):
@@ -522,8 +528,7 @@ class Widget(view.Renderable):
         around.
         """
         kw = self.prepare_dict(value, kw)
-        if self.is_root:
-            tw.framework.register_resources(self.retrieve_resources())
+        self.register_resources()
         return super(Widget, self).display(**kw)
 
     def __call__(self, value=None, **kw):
@@ -533,12 +538,16 @@ class Widget(view.Renderable):
         """Returns the default value for the widget. If the default is a funtion
         that it can be called without arguments it will be called on each
         render to retrieve a value"""
-        if callable_wo_args(self.default):
+        try:
             value = self.default()
-        else:
+        except TypeError:
             value = self.default
         return value
 
+    @property
+    def children_deep(self):
+        return self.children
+
     def prepare_dict(self, value, d, adapt=True):
         """
         Prepares the all kw arguments sent to `display` or `render` before
@@ -616,25 +625,20 @@ class Widget(view.Renderable):
             'overriden'
 
         """
-
-
-
         # Populate dict with attrs from self listed at params
-        # XXX: Avoid copying mutables provided by properties as we should
-        #      asume they're dinamic.
-        for k in self.params:
-            if k not in d:
-                attr = getattr(self,k,None)
-                if isinstance(attr, (list,dict)):
+        for k in ifilterfalse(d.__contains__, self.params):
+            attr = getattr(self, k, None)
+            if attr is not None:
+                if isinstance(attr, (list, dict)):
                     attr = copy(attr)
-                d[k] = attr
-
-        # Variables that are callable with no args are automatically
-        # called here
-        for k,v in d.iteritems():
-            if not isinstance(v,Widget) and callable_wo_args(v): 
-                log.debug("Autocalling param '%s'", k)
-                d[k] = v()
+                # Variables that are callable with no args are automatically
+                # called here
+                elif not isinstance(attr, Widget):
+                    try:
+                        attr = attr()
+                    except (TypeError, AttributeError):
+                        pass
+            d[k] = attr
 
 
 
