@@ -107,8 +107,9 @@ class EngineManager(dict):
         for ep in iter_entry_points("python.templating.engines"):
             try:
                 self.load_engine(ep.name, engine_options, stdvars)
-            except StandardError, e:
-                log.warn("Failed to load '%s' template engine: %s", ep.name, e)
+            except:
+                log.warn("Failed to load '%s' template engine: %r",
+                         ep.name, sys.exc_info())
                 if raise_error:
                     raise
 
